# -*- coding: utf-8 -*-
from django.db import models, migrations
import django.utils.timezone

from . import storage


class Migration(migrations.Migration):

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="UploadedFile",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                ("created", models.DateTimeField(default=django.utils.timezone.now)),
                (
                    "uploaded_file",
                    models.FileField(
                        storage=storage, max_length=255, upload_to=b"temp_uploads"
                    ),
                ),
                ("original_filename", models.CharField(max_length=255)),
                ("field_name", models.CharField(max_length=255, null=True, blank=True)),
                ("file_id", models.CharField(max_length=40)),
                ("form_id", models.CharField(max_length=40)),
            ],
            options={},
            bases=(models.Model,),
        ),
    ]



import json
import logging

from django.http import JsonResponse
from django.views.decorators.http import require_POST, require_http_methods, require_GET

from .utils import get_bucket_name, file_form_upload_dir, get_client, get_available_name


logger = logging.getLogger(__name__)


@require_POST
def create_upload(request):
    logger.info("Create upload")

    client = get_client()
    json_body = json.loads(request.body)
    filename = json_body["filename"]
    s3_upload_dir = json_body["s3UploadDir"]
    bucket_name = get_bucket_name()
    key = get_available_name(
        client,
        bucket_name,
        file_form_upload_dir()
        + ("/" + s3_upload_dir if s3_upload_dir else "")
        + "/"
        + filename,
    )
    content_type = json_body["contentType"]
    response = client.create_multipart_upload(
        Bucket=bucket_name,
        Key=key,
        ContentType=content_type,
    )
    return JsonResponse({"key": response["Key"], "uploadId": response["UploadId"]})


@require_http_methods(["DELETE"])
def abort_upload(request, upload_id):
    logger.info("Abort upload")
    client = get_client()
    bucket_name = get_bucket_name()
    key = request.GET["key"]

    client.abort_multipart_upload(
        Bucket=bucket_name,
        Key=key,
        UploadId=upload_id,
    )
    return JsonResponse({})


@require_GET
def sign_upload_part(request, upload_id, part_number):
    logger.info("Generate presigned url")

    client = get_client()
    key = request.GET["key"]
    bucket_name = get_bucket_name()
    response = client.generate_presigned_url(
        ClientMethod="upload_part",
        Params={
            "Bucket": bucket_name,
            "Key": key,
            "UploadId": upload_id,
            "Body": "",
            "PartNumber": part_number,
        },
        ExpiresIn=3600,
    )
    return JsonResponse({"url": response})


@require_POST
def complete_upload(request, upload_id):
    logger.info("Complete upload")

    client = get_client()
    json_body = json.loads(request.body)
    key = request.GET["key"]
    parts = json_body["parts"]
    bucket_name = get_bucket_name()
    response = client.complete_multipart_upload(
        Bucket=bucket_name,
        Key=key,
        UploadId=upload_id,
        MultipartUpload={"Parts": parts},
    )
    return JsonResponse({"location": response["Location"]})



import os
from pathlib import Path
from django.conf import settings
from django.core.cache import caches
from django.core.files import File
from django.http import HttpResponse
from django.core.files.storage import FileSystemStorage

from django_file_form.models import TemporaryUploadedFile, storage_class
from django_file_form import conf


cache = caches[getattr(settings, "FILE_FORM_CACHE", "default")]


def remove_resource_from_cache(resource_id):
    cache.delete_many(
        [
            "tus-uploads/{}/file_size".format(resource_id),
            "tus-uploads/{}/filename".format(resource_id),
            "tus-uploads/{}/offset".format(resource_id),
            "tus-uploads/{}/metadata".format(resource_id),
        ]
    )


def create_uploaded_file_in_db(
    field_name, file_id, form_id, original_filename, uploaded_file
):
    values = dict(
        file_id=file_id,
        form_id=form_id,
        original_filename=original_filename,
    )

    if field_name:
        values["field_name"] = field_name

    if storage_class == FileSystemStorage and not getattr(
        settings, "FILE_FORM_ALWAYS_COPY_UPLOADED_FILE", False
    ):
        create_uploaded_file_in_db_with_move(values, uploaded_file)
    else:
        create_uploaded_file_in_db_with_copy(values, uploaded_file)


def create_uploaded_file_in_db_with_copy(values, uploaded_file):
    with open(uploaded_file, "rb") as fh:
        values["uploaded_file"] = File(file=fh, name=uploaded_file.name)
        TemporaryUploadedFile.objects.create(**values)

    os.remove(uploaded_file)


def create_uploaded_file_in_db_with_move(values, uploaded_file):
    values["uploaded_file"] = str(uploaded_file.relative_to(Path(settings.MEDIA_ROOT)))

    TemporaryUploadedFile.objects.create(**values)


tus_api_version = "1.0.0"
tus_api_version_supported = [
    "1.0.0",
]
tus_api_extensions = ["creation", "termination", "file-check"]


def get_tus_response():
    response = HttpResponse()
    response["Tus-Resumable"] = tus_api_version
    response["Tus-Version"] = ",".join(tus_api_version_supported)
    response["Tus-Extension"] = ",".join(tus_api_extensions)
    response["Access-Control-Allow-Origin"] = "*"
    response["Access-Control-Allow-Methods"] = "PATCH,HEAD,GET,POST,OPTIONS"
    response[
        "Access-Control-Expose-Headers"
    ] = "Tus-Resumable,upload-length,upload-metadata,Location,Upload-Offset"
    response[
        "Access-Control-Allow-Headers"
    ] = "Tus-Resumable,upload-length,upload-metadata,Location,Upload-Offset,content-type"
    response["Cache-Control"] = "no-store"

    return response



# Generated by Django 2.2.28 on 2022-05-19 07:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_file_form', '0007_auto_20210119_0104'),
    ]

    operations = [
        migrations.AlterField(
            model_name='temporaryuploadedfile',
            name='id',
            field=models.AutoField(primary_key=True, serialize=False),
        ),
    ]



# Generated by Django 3.0.5 on 2020-05-01 14:08

from django.db import migrations, models
import django_file_form.models

from . import storage


class Migration(migrations.Migration):

    dependencies = [
        ("django_file_form", "0005_auto_20200407_0814"),
    ]

    operations = [
        migrations.AlterField(
            model_name="uploadedfile",
            name="uploaded_file",
            field=models.FileField(
                max_length=255,
                storage=storage,
                upload_to=django_file_form.models.get_upload_to,
            ),
        ),
    ]



from django.core.management.base import BaseCommand

from django_file_form.models import TemporaryUploadedFile


class Command(BaseCommand):
    help = "Removes temporary uploaded files and records. Deletes if the file is created at least one day ago."

    def handle(self, *args, **options):
        deleted_files = TemporaryUploadedFile.objects.delete_unused_files()

        verbosity = options.get("verbosity")
        if verbosity:
            if not deleted_files:
                print("No files deleted")
            else:
                print("Deleted files: {0!s}".format(", ".join(deleted_files)))



from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("django_file_form", "0006_auto_20200501_0908"),
    ]

    operations = [
        migrations.RenameModel(
            new_name="TemporaryUploadedFile",
            old_name="UploadedFile",
        ),
        migrations.AlterModelTable(
            name="TemporaryUploadedFile",
            table="django_file_form_uploadedfile",
        ),
    ]



from pathlib import Path
from django.core.exceptions import PermissionDenied
from django.conf import settings


def check_permission(request):
    must_login = getattr(settings, "FILE_FORM_MUST_LOGIN", False)

    if must_login and not request.user.is_authenticated:
        raise PermissionDenied()


def get_upload_path():
    default_upload_dir = "temp_uploads"
    upload_path = Path(getattr(settings, "FILE_FORM_UPLOAD_DIR", default_upload_dir))

    if upload_path.is_absolute():
        return upload_path
    else:
        return Path(settings.MEDIA_ROOT).joinpath(upload_path)



from django.db import models


class ModelManager(models.Manager):
    def try_get(self, **kwargs):
        qs = self.get_queryset().filter(**kwargs)
        if qs.exists():
            return qs.get()
        else:
            return None



import os
import uuid
from typing import Union, List, Dict

from django.core.files import uploadedfile
from django.core.exceptions import ImproperlyConfigured
from django.db.models.fields.files import FieldFile


class PlaceholderUploadedFile:
    def __init__(self, name, file_id=None, size=None, metadata=None):
        self.name = name
        self.file_id = file_id or uuid.uuid4().hex
        if size is None:
            self.size = os.path.getsize(self.name)
        else:
            self.size = size

        self.is_placeholder = True
        self.is_s3direct = False
        self.metadata = metadata

    def get_initial_data(self):
        return dict(id=self.file_id, name=self.name, size=self.size, type="placeholder")


class UploadedTusFile(uploadedfile.UploadedFile):
    def __init__(self, file_id, metadata=None, **kwargs):
        super().__init__(**kwargs)

        self.file_id = file_id
        self.size = self.file.size

        self.is_placeholder = False
        self.is_s3direct = False
        self.metadata = metadata

    def get_initial_data(self):
        return dict(id=self.file_id, name=self.name, size=self.size, type="tus")


try:
    from storages.backends.s3boto3 import S3Boto3Storage, S3Boto3StorageFile
    from storages.utils import setting

    class S3UploadedFileWithId(S3Boto3StorageFile):
        def __init__(self, file_id, name, original_name, size, metadata=None, **kwargs):
            boto_storage = S3Boto3Storage(
                bucket_name=setting("AWS_STORAGE_BUCKET_NAME"),
                endpoint_url=setting("AWS_S3_ENDPOINT_URL"),
                access_key=setting(
                    "AWS_S3_ACCESS_KEY_ID", setting("AWS_ACCESS_KEY_ID")
                ),
                secret_key=setting(
                    "AWS_S3_SECRET_ACCESS_KEY", setting("AWS_SECRET_ACCESS_KEY")
                ),
            )

            super().__init__(name=name, mode="rb", storage=boto_storage, **kwargs)

            self.file_id = file_id
            self.original_name = original_name
            # self.size is derived from S3boto3StorageFile
            # but size is passed for consistency, and potentially
            # for validation
            self.is_placeholder = False
            self.is_s3direct = True
            self.metadata = metadata

        def get_initial_data(self):
            return dict(id=self.file_id, name=self.name, size=self.size, type="s3")


except (ImportError, ImproperlyConfigured):
    # S3 is an optional feature but we keep the symbol
    class S3UploadedFileWithId:
        pass


UploadedFileTypes = Union[FieldFile, PlaceholderUploadedFile, UploadedTusFile]

UploadedFileTypesOrList = Union[
    UploadedFileTypes,
    List[Union[UploadedFileTypes]],
]


def get_initial_data_from_field_file(field_file: FieldFile) -> Dict:
    try:
        return dict(name=field_file.name, size=field_file.size, type="existing")
    except:
        return dict()


def get_initial_data_from_uploaded_file(uploaded_file: UploadedFileTypes) -> Dict:
    if isinstance(uploaded_file, FieldFile):
        return get_initial_data_from_field_file(uploaded_file)
    else:
        return uploaded_file.get_initial_data()



from django.forms import FileField
from django.forms.widgets import FILE_INPUT_CONTRADICTION
from django.core import validators

from .widgets import UploadWidget, UploadMultipleWidget
from .models import TemporaryUploadedFile
from .util import get_list


class UploadedFileField(FileField):
    widget = UploadWidget

    def __init__(self, *, accept=None, **kwargs):
        self.accept = accept

        super().__init__(**kwargs)

    # override

    def widget_attrs(self, widget):
        if self.accept:
            return dict(accept=self.accept)
        else:
            return dict()

    # new methods

    def delete_file_data(self, field_name, form_id):
        for f in TemporaryUploadedFile.objects.for_field_and_form(field_name, form_id):
            f.delete()

    def get_file_data(self, field_name, form_id):
        try:
            return (
                TemporaryUploadedFile.objects.for_field_and_form(field_name, form_id)
                .latest("created")
                .get_uploaded_file()
            )
        except TemporaryUploadedFile.DoesNotExist:
            return None

    def bound_data(self, data, initial):
        if data in (None, FILE_INPUT_CONTRADICTION):
            return initial
        return data


class MultipleUploadedFileField(UploadedFileField):
    widget = UploadMultipleWidget

    # override

    # These overrides are necessary to support multiple files. The FileField from Django doesn't support multiple files.

    def bound_data(self, data, initial):
        result = []

        if initial:
            result += get_list(initial)

        if data:
            result += get_list(data)

        return result

    def clean(self, data, initial=None):
        # Parameters:
        # * data: new data
        # * initial: initial data
        #
        # The original behaviour is to return either the new data or the initial data.
        # The new behaviour is to return the new data plus the initial data.
        return super().clean((data or []) + (initial or []))

    def to_python(self, data):
        if data in validators.EMPTY_VALUES:
            return None
        elif isinstance(data, list):
            return [super(MultipleUploadedFileField, self).to_python(f) for f in data]
        else:
            return [data]

    def widget_attrs(self, widget):
        attrs = super().widget_attrs(widget)

        attrs["multiple"] = "multiple"
        return attrs

    # new methods

    def get_file_data(self, field_name, form_id):
        temporary_uploaded_files = TemporaryUploadedFile.objects.for_field_and_form(
            field_name, form_id
        )

        return [f.get_uploaded_file() for f in temporary_uploaded_files]



import base64
import logging
import uuid
import json

from django.core.exceptions import PermissionDenied
from django.http import HttpResponseForbidden
from django.views.decorators.http import require_POST, require_http_methods
from django.utils._os import safe_join, to_path

from django_file_form import conf
from django_file_form.models import TemporaryUploadedFile
from django_file_form.django_util import get_upload_path, check_permission
from .utils import (
    cache,
    create_uploaded_file_in_db,
    get_tus_response,
    remove_resource_from_cache,
)


logger = logging.getLogger(__name__)


@require_POST
def start_upload(request):
    try:
        check_permission(request)
    except PermissionDenied:
        return HttpResponseForbidden(
            json.dumps(
                dict(status="permission denied")
            ),
            content_type="application/json"
        )

    response = get_tus_response()

    if request.META.get("HTTP_TUS_RESUMABLE", None) is None:
        logger.warning("Received File upload for unsupported file transfer protocol")
        response.status_code = 500
        response.reason_phrase = (
            "Received File upload for unsupported file transfer protocol"
        )
        return response

    metadata = {}
    upload_metadata = request.META.get("HTTP_UPLOAD_METADATA", None)

    if upload_metadata:
        for kv in upload_metadata.split(","):
            (key, value) = kv.split(" ")
            metadata[key] = base64.b64decode(value).decode("utf-8")

    logger.info(f"TUS post metadata={metadata}")

    file_size = int(request.META.get("HTTP_UPLOAD_LENGTH", "0"))
    resource_id = str(uuid.uuid4())

    cache.add(
        "tus-uploads/{}/filename".format(resource_id),
        metadata.get("filename"),
        conf.CACHE_TIMEOUT,
    )
    cache.add(
        "tus-uploads/{}/file_size".format(resource_id), file_size, conf.CACHE_TIMEOUT
    )
    cache.add("tus-uploads/{}/offset".format(resource_id), 0, conf.CACHE_TIMEOUT)
    cache.add(
        "tus-uploads/{}/metadata".format(resource_id), metadata, conf.CACHE_TIMEOUT
    )

    try:
        with to_path(safe_join(get_upload_path(), resource_id)).open("wb") as f:
            pass
    except IOError as e:
        logger.error(
            "Unable to create file: {}".format(e),
            exc_info=True,
            extra={
                "request": request,
            },
        )
        response.status_code = 500
        return response

    response.status_code = 201
    response["Location"] = "{}{}".format(request.get_full_path(), resource_id)
    response["ResourceId"] = resource_id
    return response


@require_http_methods(["DELETE", "HEAD", "PATCH"])
def handle_upload(request, resource_id):
    check_permission(request)

    if request.method == "DELETE":
        return cancel_upload(resource_id)
    if request.method == "HEAD":
        return upload_info(resource_id)
    elif request.method == "PATCH":
        return upload_part(request, resource_id)


def upload_info(resource_id):
    logger.info(f"TUS head resource_id={resource_id}")

    response = get_tus_response()

    offset = cache.get("tus-uploads/{}/offset".format(resource_id))
    file_size = cache.get("tus-uploads/{}/file_size".format(resource_id))
    if offset is None:
        logger.info("TUS head resource not found")
        response.status_code = 404
        return response

    else:
        response.status_code = 200
        response["Upload-Offset"] = offset
        response["Upload-Length"] = file_size

    return response


def upload_part(request, resource_id):
    response = get_tus_response()

    filename = cache.get("tus-uploads/{}/filename".format(resource_id))
    metadata = cache.get("tus-uploads/{}/metadata".format(resource_id))
    offset = cache.get("tus-uploads/{}/offset".format(resource_id))

    file_offset = int(request.META.get("HTTP_UPLOAD_OFFSET", 0))
    chunk_size = int(request.META.get("CONTENT_LENGTH", 102400))

    upload_file_path = to_path(safe_join(get_upload_path(), resource_id))
    if filename is None or not upload_file_path.exists():
        response.status_code = 410
        return response

    if file_offset != offset:  # check to make sure we're in sync
        response.status_code = 409  # HTTP 409 Conflict
        return response

    logger.info(
        f"TUS patch resource_id={resource_id} filename={filename} metadata={metadata} offset={offset} upload_file_path={upload_file_path}"
    )

    try:
        file = upload_file_path.open("r+b")
    except IOError:
        file = upload_file_path.open("wb")

    if file:
        try:
            file.seek(file_offset)
            file.write(request.body)
        except IOError:
            response.status_code = 500
            return response
        finally:
            file.close()

    try:
        new_offset = cache.incr("tus-uploads/{}/offset".format(resource_id), chunk_size)
    except ValueError:
        response.status_code = 404
        return response

    file_size_string = cache.get("tus-uploads/{}/file_size".format(resource_id))

    if file_size_string is None:
        response.status_code = 404
        return response

    response["Upload-Offset"] = new_offset
    response.status_code = 204

    file_size = int(file_size_string)

    if file_size == new_offset:
        remove_resource_from_cache(resource_id)

        create_uploaded_file_in_db(
            field_name=metadata.get("fieldName"),
            file_id=resource_id,
            form_id=metadata.get("formId"),
            original_filename=metadata.get("filename"),
            uploaded_file=upload_file_path,
        )

    return response


def cancel_upload(resource_id):
    logger.info(f"TUS delete resource_id={resource_id}")

    remove_resource_from_cache(resource_id)

    upload_file_path = to_path(safe_join(get_upload_path(), resource_id))

    if upload_file_path.exists():
        upload_file_path.unlink()

    uploaded_file = TemporaryUploadedFile.objects.try_get(file_id=resource_id)

    if uploaded_file:
        uploaded_file.delete()

    response = get_tus_response()
    response.status_code = 204
    return response



from typing import Type, TYPE_CHECKING, TypeVar


T = TypeVar("T")


def with_typehint(baseclass: Type[T]) -> Type[T]:
    """
    Useful function to make mixins with baseclass typehint

    ```
    class ReadonlyMixin(with_typehint(BaseAdmin))):
        ...
    ```
    """
    if TYPE_CHECKING:
        return baseclass

    return object



import json
import uuid

from django.urls import reverse
from django.forms import CharField, Form, HiddenInput

from .util import get_list
from .type_util import with_typehint
from .uploaded_file import get_initial_data_from_uploaded_file

# UploadedFileField and MultipleUploadedFileField must be in this module because they are in the api
from .fields import UploadedFileField, MultipleUploadedFileField


class FileFormMixin(with_typehint(Form)):
    s3_upload_dir = None

    def __init__(self, *args, **kwargs):
        s3_upload_dir = kwargs.pop("s3_upload_dir", self.s3_upload_dir)

        super().__init__(*args, **kwargs)

        self._update_files_data()

        if s3_upload_dir is not None:
            self._add_hidden_field("s3_upload_dir", s3_upload_dir)
            self._add_hidden_field("upload_url", reverse("s3_upload"))
        else:
            self._add_hidden_field("upload_url", reverse("tus_upload"))

        self._add_hidden_field("form_id", uuid.uuid4())
        self._add_upload_inputs()
        self._add_metadata_inputs()

    # override

    def get_initial_for_field(self, field, field_name: str):
        initial = super().get_initial_for_field(field, field_name)

        if not hasattr(field, "get_file_data"):
            return initial

        placeholder_field_name = self.add_prefix(field_name) + "-uploads"
        file_uploads_value = self.data.get(placeholder_field_name)

        if not file_uploads_value:
            return initial

        file_info_list = json.loads(file_uploads_value)
        existing_filenames_in_uploads = {
            file_info["name"]
            for file_info in file_info_list
            if file_info["type"] == "existing"
        }

        if isinstance(initial, list):
            return [
                initial_file
                for initial_file in initial
                if initial_file.name in existing_filenames_in_uploads
            ]
        else:
            if existing_filenames_in_uploads:
                return initial
            else:
                return None

    # new methods

    def delete_temporary_files(self):
        form_id = self.data.get(self.add_prefix("form_id"))

        if not form_id:
            return

        for field_name, field in self.fields.items():
            if hasattr(field, "delete_file_data"):
                prefixed_field_name = self.add_prefix(field_name)
                field.delete_file_data(prefixed_field_name, form_id)

    # private

    def _add_hidden_field(self, name: str, initial):
        self.fields[name] = CharField(
            widget=HiddenInput, initial=initial, required=False
        )

    def _add_metadata_inputs(self):
        for field_name in self._file_form_field_names():
            self._add_hidden_field(
                f"{field_name}-metadata",
                json.dumps(self._get_metadata_for_field(field_name)),
            )

    def _add_upload_inputs(self):
        for field_name in self._file_form_field_names():
            self._add_hidden_field(
                f"{field_name}-uploads",
                json.dumps(self._get_upload_data_for_field(field_name)),
            )

    def _file_form_field_names(self):
        return [
            field_name
            for field_name, field in self.fields.items()
            if hasattr(field, "get_file_data")
        ]

    def _get_metadata_for_field(self, field_name: str):
        initial_values = get_list(self.initial.get(field_name, []))

        return {
            value.name: value.metadata
            for value in initial_values
            if hasattr(value, "metadata")
        }

    def _get_upload_data_for_field(self, field_name: str):
        uploaded_files = get_list(self.initial.get(field_name, []))

        return [
            get_initial_data_from_uploaded_file(uploaded_file)
            for uploaded_file in uploaded_files
        ]

    def _update_files_data(self):
        form_id = self.data.get(self.add_prefix("form_id"))

        if not form_id:
            return

        for field_name in self._file_form_field_names():
            field = self.fields[field_name]
            prefixed_field_name = self.add_prefix(field_name)

            file_data = field.get_file_data(prefixed_field_name, form_id)

            if file_data:
                # NB: django-formtools wizard uses dict instead of MultiValueDict
                if isinstance(file_data, list) and hasattr(self.files, "setlist"):
                    self.files.setlist(prefixed_field_name, file_data)
                else:
                    self.files[prefixed_field_name] = file_data



from setuptools import setup, find_packages

from os import path
from io import open

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

version = "3.5.4"

setup(
    name="django-file-form",
    version=version,
    packages=find_packages(),
    license="Apache License, Version 2.0",
    long_description=long_description,
    long_description_content_type="text/markdown",
    include_package_data=True,
    zip_safe=False,
    author="Marco Braak",
    author_email="marco.braak@planet.nl",
    description="Django-file-form helps you to write forms with a pretty ajax upload",
    url="https://github.com/mbraak/django-file-form",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Framework :: Django",
        "Framework :: Django :: 3.2",
        "Framework :: Django :: 4.1",
        "Framework :: Django :: 4.2",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: Implementation :: CPython",
    ],
)



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-04-23 08:29
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("django_file_form", "0003_auto_20170317_1230"),
    ]

    operations = [
        migrations.AlterIndexTogether(
            name="uploadedfile",
            index_together=set([("form_id", "field_name")]),
        ),
    ]
